unit UniLabel;

{
   UniLabel.pas Copyright (C) 1999, 2000 David Mason

   This code is freeware:

        - It may be used by anyone, private or commercial.
        - It may be freely distributed without modification.

        - Do not modify and redistribute without checking
          with the author.
        - Do not remove this notice.
        - If you find it useful, please email me:
          drmason@xtra.co.nz
        - If this component is used in a program that
          makes you lots of money, please consider
          donating some to the author.

   THIS CODE IS PROVIDED AS IS, WITH NO GUARANTEES.
   USE THIS CODE AT YOUR OWN RISK. THE AUTHOR WILL
   NOT BE HELD RESPONSIBLE FOR ANY DAMAGE CAUSED BY
   THIS CODE.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, UniDefs;

type
  TCustomUniLabel = class(TCustomLabel)
  private
    procedure CMTextChanged(var Message: TMessage); message CM_TEXTCHANGED;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure ConvertCaption;
    procedure SizeCaption;
    procedure SetCodePage(value:TCodePage);
  protected
    FCodePage:TCodePage;
    FUnicodeCaption:PWideChar;
    FCaptionLength:word;
    FCaptionSize:TSize;
    FItalicSpace:integer;
    procedure DoDrawText(var Rect: TRect;Flags: Longint); override;
    procedure Loaded; override;

    property CodePage : TCodePage read FCodePage write SetCodePage;
  public
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
  end;

  TUniLabel = class(TCustomUniLabel)
  published
    property CodePage;
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property Caption;
    property Color;
    property Constraints;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

implementation

{ TCustomUniLabel }

procedure TCustomUniLabel.CMFontChanged(var Message: TMessage);
begin
  SizeCaption;
  inherited;
end;

procedure TCustomUniLabel.CMTextChanged(var Message: TMessage);
begin
  ConvertCaption;
  SizeCaption;
  inherited;
end;

procedure TCustomUniLabel.ConvertCaption;
begin
  ConvertToUnicode(CodePage, Caption, FUnicodeCaption);
  FCaptionLength:=LStrLenW(FUnicodeCaption);
end;

constructor TCustomUniLabel.Create(AOwner: TComponent);
begin
  inherited;
  FCodePage:=cpANSI; // Default to ANSI code page
  FUnicodeCaption:=nil;
  ConvertCaption;
end;

destructor TCustomUniLabel.Destroy;
begin
  // Deallocate the Unicode caption
  if FUnicodeCaption<>nil then begin
    FreeMem(FUnicodeCaption);
    FUnicodeCaption:=nil;
  end;
  inherited;
end;

procedure TCustomUniLabel.Loaded;
begin
  SizeCaption;
  inherited;
end;

procedure TCustomUniLabel.DoDrawText(var Rect: TRect;Flags: Longint);
var x,y,w:integer;
begin
  if FCaptionLength>0 then begin

    if Flags and DT_CALCRECT >0 then begin
        SizeCaption;
        Rect.Right:=Rect.Left+FCaptionSize.cx + FItalicSpace;
        Rect.Bottom:=Rect.Top+FCaptionSize.cy;
    end else begin
      Canvas.Font:=Font;

      // Work out where to start writing
      x:=Rect.Left; y:=Rect.Top;
      if Alignment<>taLeftJustify then begin
        w:=Rect.Right-Rect.Left;
        case Alignment of
          taRightJustify: x:=x+w-FCaptionSize.cx;
          taCenter:       x:=x+(w - FCaptionSize.cx) div 2;
        end;
        Dec(x, FItalicSpace);
      end;

      // Write the text
      if not Enabled then begin
        Canvas.Font.Color := clBtnHighlight;
        TextOutW(Canvas.Handle, x+1,y+1,FUnicodeCaption,FCaptionLength);
        Canvas.Font.Color := clBtnShadow;
      end;
      TextOutW(Canvas.Handle, x,y,FUnicodeCaption, FCaptionLength);
    end;
  end;
end;


procedure TCustomUniLabel.SetCodePage(value: TCodePage);
begin
  if FCodePage<>value then begin
    FCodePage:=value;
    ConvertCaption;
    SizeCaption;
    Invalidate;
    AdjustBounds;
  end;
end;

procedure TCustomUniLabel.SizeCaption;
begin
  Canvas.Font:=Font;

  GetTextExtentPoint32W(Canvas.Handle, FUnicodeCaption, FCaptionLength, FCaptionSize);
  if fsItalic in Font.Style then
    // GetTextExtentPoint32W underestimates for italics sometimes
    FItalicSpace:=Canvas.TextWidth('0')
  else
    FItalicSpace:=0;
end;

end.
